Package: NetSAM
Type: Package
Title: Network Seriation And Modularization
Version: 1.38.0
Date: 2021-05-15
Author: Jing Wang <jing.wang@bcm.edu>
Maintainer: Zhiao Shi <zhiao.shi@gmail.com>
Description: The NetSAM (Network Seriation and Modularization) package takes an edge-list representation of a weighted or unweighted network as an input, performs network seriation and modularization analysis, and generates as files that can be used as an input for the one-dimensional network visualization tool NetGestalt (http://www.netgestalt.org) or other network analysis. The NetSAM package can also generate correlation network (e.g. co-expression network) based on the input matrix data, perform seriation and modularization analysis for the correlation network and calculate the associations between the sample features and modules or identify the associated GO terms for the modules.
License: LGPL
LazyLoad: yes
Depends: R (>= 3.0.0), seriation (>= 1.0-6), igraph (>= 0.6-1), tools
        (>= 3.0.0), WGCNA (>= 1.34.0), biomaRt (>= 2.18.0)
Imports: methods, AnnotationDbi (>= 1.28.0), doParallel (>= 1.0.10),
        foreach (>= 1.4.0), survival (>= 2.37-7), GO.db (>= 2.10.0),
        R2HTML (>= 2.2.0), DBI (>= 0.5-1)
Suggests: RUnit, BiocGenerics, org.Sc.sgd.db, org.Hs.eg.db,
        org.Mm.eg.db, org.Rn.eg.db, org.Dr.eg.db, org.Ce.eg.db,
        org.Cf.eg.db, org.Dm.eg.db, org.At.tair.db, rmarkdown, knitr,
        markdown
Collate: zzz.R NetSAM.R NetAnalyzer.R MatNet.R MatSAM.R consensusNet.R
        mapToSymbol.R testFileFormat.R featureAssociation.R
        GOAssociation.R mergeDuplicate.R
NeedsCompilation: no
VignetteBuilder: knitr
Packaged: 2022-11-01 22:11:57 UTC; biocbuild
git_url: https://git.bioconductor.org/packages/NetSAM
git_branch: RELEASE_3_16
git_last_commit: dd9a6db
git_last_commit_date: 2022-11-01
Date/Publication: 2022-11-01
